//
// This creates a python script "datafile.py", using the data
// in "datafile", from which a graph may be displayed, using
// the command "python datafile.py".  (Here "datafile" is
// generic notation for the name of the data file.)  This
// code, at its end, issues the command "python datafile.py",
// which should display the graph on the terminal screen.
//
// The python script actually reads the data from a temporary
// file (tfn), into which only the data is put, without any
// comments or other lines of text, so as to ensure that the
// python script works correctly.
//

#include "plotdata.h"

void mk_graph(string fname, opts& o)                     // fname: datafile
{
  int k;
  dvec x;
  dvec y;
  dvec s;
  ifstream ifs;
  ofstream ofs;

  time_t now = time(NULL);              // get the time/date string into td
  struct tm* info = localtime(&now);
  cstring fmt = "%I:%M %p on %b %d, %Y";
  string td;
  gstrftime(td, fmt, info);

  gopen(ifs, fname);                               // open datafile (fname)
  assert(ifs);
  get_data(ifs, x, y, s);        // get the data from datafile into x,y,[s]

  string tfn = fname + ".tmp";                  // tfn: temporary file name

  gopen(ofs, tfn);                                              // open tfn
  assert(ofs);
  for (k=0; k < x.size(); ++k) {              // put just the data into tfn
    ofs << x[k] << "\t" << y[k];
    if (s.size()) {
      ofs << "\t" << s[k] << "\n";
    }
    else {
      ofs << "\n";
    }
  }
  ofs.close();

  string pyfn = fname + ".py";       // pyfn = "fname.py" (python filename)
  string cmd = "python " + pyfn;                // command to display graph
  dbl xmin;
  dbl xmax;
  int xlim1;                                          // for extending axes
  int xlim2;
  xmin = numeric_limits<dbl>::max();                      // largest double
  xmax = numeric_limits<dbl>::min();                     // smallest double
  for (k = 0; k < x.size(); ++k) {
    xmin = (x[k] <= xmin) ? x[k] : xmin;
    xmax = (x[k] >= xmax) ? x[k] : xmax;
  }
  xlim1 = (xmin*(1.1) - 0.1*xmax) + 0.5;            // for extending x-axis
  xlim2 = (xmax*(1.1) - 0.1*xmin) + 0.5;            // for extending x-axis
  if (s.size() && !o.sigmas) {
    cerr << " Note:\n"
         << "  I see you have included a third column of uncertainties\n"
         << "  in your datafile, but have not specified the \033[1m--sigmas\033[0m\n"
         << "  option, so we will ignore your third column of entries when\n"
         << "  plotting your data.  If you have provided independently\n"
         << "  estimated uncertainties, re-run this program with the\n"
         << "  \033[1m--sigmas\033[0m option specified, so that they will be taken\n"
         << "  into account, with error bars appearing on your plot.\n\n";
  }
  if (!s.size() && o.sigmas) {
    err("  You have specified the \033[1m--sigmas\033[0m option, but there are no\n \
    values of sigma listed in the third column of your datafile.\n \
    Try re-running this program without the \033[1m--sigmas\033[0m option.");
  }
  string title;
  string xlabel;
  string ylabel;
  string question;
  question = "\n  OK, please provide a brief title for\n"
             "  your graph---see \033[1mplotdata --help\033[0m for hints.\n"
             "  (Insert \" td\" to get a \"time/date\" string.)\n"
             "  Just hit \'return\' for no title.:";
  title = get_string(question, td, 2);
  cout << "  title = " << title << "\n";        // display the title string

  question = "\n  Provide also a brief x-axis label:\n"
             "  (Again hit \'return\' for no label.)";
  xlabel = get_string(question, td, 2);
  cout << "  xlabel = " << xlabel << "\n";     // display the xlabel string
  question = "\n  And also a brief y-axis label:\n"
             "  (And again hit \'return\' for no label.)";
  ylabel = get_string(question, td, 2);
  cout << "  ylabel = " << ylabel << "\n";     // display the ylabel string



  gopen(ofs, pyfn);                                        // open fname.py
  assert(ofs);
  ofs << "# A graph may be created using the command \'python file.py\'\n"
      << "# where \'file.py\' is this file.  python, numpy and matplotlib\n"
      << "# must be present on your system for this to work.  You currently\n"
      << "# have installed the versions shown below.\n"
      << "#\n"
      << "from platform import python_version\n"
      << "from pylab import *\n"
      << "print \"python version:    \",python_version()\n"
      << "print \"numpy version:     \",np.__version__\n"
      << "print \"matplotlib version:\",matplotlib.__version__\n\n"
      << "# The next line loads the data from your (temporary) datafile.\n"
      << "data=np.loadtxt(\'" << tfn << "\')\n\n"     // load data from tfn
      << "# " <<  td << "\n\n"
      << "# In the next line (which plots the data points), \'gd\' means\n"
      << "# green diamonds.  Other colors or point shapes may be substituted.\n"
      << "# If the --sigmas option is specified, and you have sigmas as a\n"
      << "# third column in your datafile, the points will have error bars.\n\n";
  if (o.sigmas && s.size()) {
    ofs << "errorbar(data[:,0], data[:,1], data[:,2], fmt='gd', capsize=4)\n";
  }
  else {
    ofs << "plot(data[:,0], data[:,1], \'gd\')\n\n";
  }
  ofs << "# This next line extends the x-axis to provide needed white space.\n"
      << "xlim(" << xlim1 << ", " << xlim2 << ")\n\n"
      << "# The next three lines of code set your title and labels for the axes.\n"
      << "# You can insert newline characters (\'\\n\') in your string if needed,\n"
      << "# and/or incorporate \"TeX\" characters by enclosing them in \'$\' signs.\n"
      << "# See the help file for hints on how to insert \"TeX\" characters.\n"
      << "title(" << title << ", fontsize=13, linespacing=1.6)\n"
      << "xlabel(" << xlabel << ", linespacing=1.6)\n"
      << "ylabel(" << ylabel << ", linespacing=1.6)\n"
      << "# Additional lines of text may be placed anywhere on your graph by\n"
      << "# editing this file after the program ends.  Here's a template:\n"
      << "# text(X, Y, r'$T^2 =\\alpha M + \\beta$, is plotted', fontsize=13)\n"
      << "# Here (X,Y) is the lower left position of the text string in user units,\n"
      << "# and characters between the \'$\' signs are \"TeX\" math characters.\n"
      << "# The initial \'r\' will be needed if you insert \"TeX\" characters.\n\n"
      << "# Now we put a grid on the plot, and issue the \"show()\" command to\n"
      << "# display the plot on your screen.\n"
      << "tight_layout()\n"
      << "grid()\n"
      << "show()\n";
  ofs.close();
  gsystem(cmd);
  question = "\n  Would you like to remove \"" + tfn + "\"?  (Say no\n"
             "  if you want to edit the python script.) [yes],no";
  string answer = get_string(question, td, 0);
  if (answer[0] == '\0' || answer[0] == 'y' || answer[0] == 'Y') {
    remove(tfn.c_str());
  }
  cout << "\n";
}

//
// This function asks a question,
// then gets and returns an answer.
// If k=2, and the string submitted contains " td",
// a time/date string will exist in the returned string,
// and the returned string will be enclosed in single quotes.
// See above for examples.
//
string get_string(cstring question, cstring td, uns k)
{
  string line;
  if (k) cout << question << "\n";
  else cout << question << " ";
  getline(cin, line);
  if (k==2) {                               // for title, xlabel and ylabel
    if (line.find(" td") != string::npos) {
      line.replace(line.find("td"),2,td);           // td: time/date string
    }
    line = "\'" + line + "\'";    // enclose by "'" chars, needed by python
  }
  return line;
}

